from boatwright import Strategy
from boatwright.Orders.Order import MarketOrder
from boatwright.Indicators import sma, crossover

class MACD(Strategy):
    """
    Moving Average Convergance Divergance (MACD) strategy
    """

    def __init__(self, fast_period:int=7, slow_period:int=14, symbol:str="BTC", strategy_id="example_MACD"):
        super().__init__(symbol, strategy_id=strategy_id)
        self.p["fast_period"] = fast_period
        self.p["slow_period"] = slow_period

    def calculate_signals(self, df):
        df["fast"] = sma(df["close"], period=self.p['fast_period'])
        df["slow"] = sma(df["close"], period=self.p['slow_period'])
        df["macd"] = df["fast"] - df["slow"]
        df["trigger"] = crossover(df["macd"], 0)
        return df
    
    def calc_prerequisite_data_length(self):
        return self.p['slow_period']
    
    def step(self, row):
        trigger = row["trigger"]
        datetime = row["datetime"]

        if trigger == 1:
            buy_order = MarketOrder(symbol=self.symbol, side="BUY", time_opened=datetime, frac=0.95)
            self.broker.place_order(buy_order)
        if trigger == -1:
            sell_order = MarketOrder(symbol=self.symbol, side="SELL", time_opened=datetime, frac=1)
            self.broker.place_order(sell_order)
    
    def plot_info(self):
        info = {
            "fast": {"ax_n": 2, "color":"C0"},
            "slow": {"ax_n": 2, "color":"C1"},
            "trigger": {"ax_n": 3, "color":"magenta"},
        }
        return info
